h1 {
    color: var(--pico-primary)
}

h2 {
    color: var(--pico-primary)
}

h3 {
    color: var(--pico-primary)
}

h4 {
    color: var(--pico-primary)
}

h5 {
    color: var(--pico-primary)
}

h6 {
    color: var(--pico-primary)
}
